package net.knarcraft.serverlauncher.server;

import javax.naming.ConfigurationException;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Scanner;

/**
 * Contains the bare minimum to be a functional server type.
 *
 * @author		Kristian Knarvik <kristian.knarvik@knett.no>
 * @version	0.0.0.1
 * @since		0.0.0.1
 */
public class ServerType {
	private final String name;
	private final String[] versions;
	private final String downloadURL;
	private static final ArrayList<ServerType> serverTypes = new ArrayList<>();
	
	ServerType(String name, String[] versions, String downloadURL) {
		this.name = name;
		this.versions = versions;
		this.downloadURL = downloadURL;
		serverTypes.add(this);
	}
	
	public String getName() {
		return this.name;
	}
	
	public String[] getVersions() {
		return this.versions;
	}

	public String getDownloadURL() {
		return this.downloadURL;
	}

	public static ArrayList<ServerType> getServerTypes() {
		return serverTypes;
	}

	public static String[] getTypeNames() {
		ArrayList<ServerType> types = ServerType.getServerTypes();
		String[] serverTypes = new String[types.size()];
		for (int i = 0; i <  types.size(); i++) {
			serverTypes[i] = types.get(i).getName();
		}
		return serverTypes;
	}

	public static ServerType getByName(String name) {
		for (ServerType serverType : serverTypes) {
			if (serverType.getName().equals(name)) {
				return serverType;
			}
		}
		return null;
	}

	/**
	 * Reads valid server types and version from a file, and creates their objects.
	 *
	 * @throws ConfigurationException if anything goes wrong.
	 */
	public static void loadServerTypes() throws ConfigurationException {
		if (serverTypes.isEmpty()) {
			Scanner file;
			try {
				file = new Scanner(new File("config/servertypes.csv"));
			} catch (FileNotFoundException e) {
				file = new Scanner(ServerType.class.getResourceAsStream("/config/servertypes.csv"));
			}

			while (file.hasNextLine()) {
                String[] str = file.nextLine().split(";", -1);
                int len = str.length;
                String[] ver;
                if (str[1].contains(",")) {
                    ver = str[1].split(",", -1);
                } else {
                    ver = new String[]{str[1]};
                }
                switch (len) {
                    case 7:
                        new AdvancedServerType(str[0], ver, str[2], str[3], str[4], str[5], str[6]);
                        break;
                    case 3:
                        new ServerType(str[0], ver, str[2]);
                        break;
                    default:
                        throw new ConfigurationException("Error: Configuration file invalid.");
                }
            }
		} else {
			throw new ConfigurationException("Error: Configuration already loaded.");
		}
	}
}